<?php
namespace de::phpdesignpatterns::example;
require_once 'Vehicle.php';

/**
 * Reprsentation eines beliebigen Autos,
 * das das Interface 'Vehicle' erfllt.
 */
class Car implements Vehicle, ArrayAccess, Countable {

    /**
     * Hersteller des Autos
     *
     * @var string
     */
    protected $manufacturer;

    /**
     * Farbe des Autos
     *
     * @var string
     */
    protected $color;

    /**
     * Kilometerstand des Autos
     *
     * @var integer
     */
    protected $milage;

    /**
     * Gibt an, ob der Motor schon gestartet wurde
     *
     * @var boolean
     */
    protected $engineStarted = false;

    /**
     * Datei mit technischen Daten
     *
     * @var string
     */
    protected $propFile = null;

    /**
     * Array um die technischen Daten zu speichern
     *
     * @var array
     */
    protected $techDetails = null;

    /**
     * Konstruktor fr ein neues Auto
     *
     * @param string $manufacturer
     * @param string $color
     * @param integer $milage
     */
    public function __construct($manufacturer, $color, $milage = 0, $propFile = null) {
        $this->manufacturer = $manufacturer;
        $this->color = $color;
        $this->milage = $milage;
        // Absoluten Pfad ermitteln
        if ($propFile !== null) {
        	$propFile = dirname(__FILE__) . '/' . $propFile;
        }
        $this->propFile = $propFile;
    }

    /**
     * Destruktor des Autos
     * Falls der Motor noch luft wird dieser ausgeschalten.
     */
    public function __destruct() {
        if ($this->engineStarted) {
        	$this->stopEngine();
        }
        $this->saveTechnicalDetails();
    }

    /**
     * Motor anlassen
     */
    public function startEngine() {
        $this->engineStarted = true;
    }

    /**
     * Einige Kilometer fahren
     *
     * @param integer $miles
     */
    public function moveForward($miles) {
        // Wenn der Motor nicht luft, kann nicht gefahren werden.
        if ($this->engineStarted !== true) {
        	return false;
        }
        // Kilometerstand erhhen
        $this->milage = $this->milage + $miles;
        return true;
    }

    /**
     * Motor wieder stoppen
     */
    public function stopEngine() {
        $this->engineStarted = false;
    }

    /**
     * Kilometerstand zurck geben
     *
     * @return integer
     */
    public function getMilage() {
        return $this->milage;
    }

    /**
     * Technische Daten aus der INI Datei lesen
     */
    protected function loadTechnicalDetails() {
        // Es existiert keine Datei
        if ($this->propFile === null) {
        	$this->techDetails = array();
        } else {
            // ini Datei laden
            $this->techDetails = parse_ini_file($this->propFile);
        }
    }

    /**
     * berprfen, ob ein Wert vorhanden ist
     *
     * @param string $offset
     * @return mixed
     */
    public function offsetExists($offset) {
        // Technische Daten laden
        if ($this->techDetails === null) {
            $this->loadTechnicalDetails();
        }
        return isset($this->techDetails[$offset]);
    }

    /**
     * Zugriff auf die technischen Daten erlauben
     *
     * @param string $offset
     * @return mixed
     */
    public function offsetGet($offset) {
        // Technische Daten laden
        if ($this->techDetails === null) {
            $this->loadTechnicalDetails();
        }
       	return $this->techDetails[$offset];
    }

    /**
     * Technische Details nderbar machen
     *
     * @param string $offset
     * @param mixed $value
     */
    public function offsetSet($offset, $value) {
        // Technische Daten laden
        if ($this->techDetails === null) {
            $this->loadTechnicalDetails();
        }
        $this->techDetails[$offset] = $value;
    }

    /**
     * Technische Details lschen
     *
     * @param string $offset
     */
    public function offsetUnset($offset) {
        // Technische Daten laden
        if ($this->techDetails === null) {
            $this->loadTechnicalDetails();
        }
        unset($this->techDetails[$offset]);
    }

    /**
     * Eintrge im virtuellen Array zhlen
     *
     * @return integer
     */
    public function count() {
        // Technische Daten laden
        if ($this->techDetails === null) {
            $this->loadTechnicalDetails();
        }
        return count($this->techDetails);
    }

    /**
     * Technische Details in die INI-Datei zurckschreiben.
     */
    protected function saveTechnicalDetails() {
        // Keine Datei definiert
        if ($this->propFile === null) {
        	return;
        }
        // Keine technischen Daten vorhanden
        if ($this->techDetails === null) {
            return;
        }
        // Genderten Inhalt der INI-Datei erzeugen
        $ini = "; Technische Daten fr {$this->manufacturer}\n";
        foreach ($this->techDetails as $property => $value) {
        	$ini .= "{$property} = \"{$value}\"\n";
        }
        $result = file_put_contents($this->propFile, $ini);
    }
}
?>